@@ -31,13 +31,13 @@ word_list <- function(word.list = NULL, and.or = c("and", "or"), is.are = FALSE,
             else {
                 out <- paste(paste(word.list[seq_len(L-1)], collapse = ", "),
                              word.list[L], sep = paste0(", ", and.or," "))
-                
+
             }
             if (is.are) out <- paste(out, "are")
             attr(out, "plural") = TRUE
         }
-        
-        
+
+
     }
     return(out)
 }
@@ -77,7 +77,7 @@ ordinal <- function(x) {
                                  "3" = "rd",
                                  "th"))
         if (sign(x) == -1) out <- paste0("-", out)
-        
+
         return(out)
     }
 }
@@ -98,11 +98,11 @@ round_df_char <- function(df, digits, pad = "0", na_vals = "") {
     nums <- vapply(df, is.numeric, logical(1))
     o.negs <- sapply(1:NCOL(df), function(x) if (nums[x]) df[[x]] < 0 else rep(FALSE, length(df[[x]])))
     df[nums] <- round(df[nums], digits = digits)
-    
+
     df[nas | infs] <- ""
-    
+
     df <- as.data.frame(lapply(df, format, scientific = FALSE, justify = "none"), stringsAsFactors = FALSE)
-    
+
     for (i in which(nums)) {
         if (any(grepl(".", df[[i]], fixed = TRUE))) {
             s <- strsplit(df[[i]], ".", fixed = TRUE)
@@ -121,16 +121,16 @@ round_df_char <- function(df, digits, pad = "0", na_vals = "") {
             })
         }
     }
-    
+
     df[o.negs & df == 0] <- paste0("-", df[o.negs & df == 0])
-    
+
     # Insert NA placeholders
     df[nas] <- na_vals
     df[infs] <- "N/A"
-    
+
     if (length(rn) > 0) rownames(df) <- rn
     if (length(cn) > 0) names(df) <- cn
-    
+
     return(df)
 }
 text_box_plot <- function(range.list, width = 12) {
@@ -149,7 +149,7 @@ text_box_plot <- function(range.list, width = 12) {
         dashes <- max_(0, diff(rescaled.range.list[[i]]) - 2)
         #|
         spaces2 <- max_(0, diff(rescaled.full.range) - (spaces1 + 1 + dashes + 1))
-        
+
         d[i, 2] <- paste0(paste(rep(" ", spaces1), collapse = ""), "|", paste(rep("-", dashes), collapse = ""), "|", paste(rep(" ", spaces2), collapse = ""))
     }
     return(d)
@@ -202,24 +202,24 @@ between <- function(x, range, inclusive = TRUE, na.action = FALSE) {
     if (!all(is.numeric(x))) stop("x must be a numeric vector.", call. = FALSE)
     if (length(range) != 2) stop("range must be of length 2.", call. = FALSE)
     if (anyNA(range) || !is.numeric(range)) stop("range must contain numeric entries only.", call. = FALSE)
-    
+
     if (range[2] < range[1]) range <- c(range[2], range[1])
-    
+
     if (anyNA(x)) {
         if (length(na.action) != 1 || !is.atomic(na.action)) stop("na.action must be an atomic vector of length 1.", call. = FALSE)
     }
     if (inclusive) out <- ifelse(is.na(x), na.action, x >= range[1] & x <= range[2])
     else out <- ifelse(is.na(x), na.action, x > range[1] & x < range[2])
-    
+
     return(out)
 }
-max_ <- function(x, na.rm = TRUE) {
-    if (!any(is.finite(x))) NA_real_
-    else max(x, na.rm = na.rm)
+max_ <- function(..., na.rm = TRUE) {
+    if (!any(is.finite(unlist(list(...))))) NA_real_
+    else max(..., na.rm = na.rm)
 }
-min_ <- function(x, na.rm = TRUE) {
-    if (!any(is.finite(x))) NA_real_
-    else min(x, na.rm = na.rm)
+min_ <- function(..., na.rm = TRUE) {
+    if (!any(is.finite(unlist(list(...))))) NA_real_
+    else min(..., na.rm = na.rm)
 }
 
 #Statistics
@@ -227,10 +227,10 @@ binarize <- function(variable, zero = NULL, one = NULL) {
     nas <- is.na(variable)
     if (!is_binary(variable[!nas])) stop(paste0("Cannot binarize ", deparse(substitute(variable)), ": more than two levels."))
     if (is.character(variable)) variable <- factor(variable)
-    
+
     variable.numeric <- as.numeric(variable)
     unique.vals <- unique(variable, nmax = 2)
-    
+
     if (is_null(zero)) {
         if (is_null(one)) {
             if (0 %in% variable.numeric) zero <- 0
@@ -244,7 +244,7 @@ binarize <- function(variable, zero = NULL, one = NULL) {
     else {
         if (zero %nin% unique.vals) stop("The argument to \"zero\" is not the name of a level of variable.", call. = FALSE)
     }
-    
+
     newvar <- setNames(ifelse(!nas & variable.numeric == zero, 0L, 1L), names(variable))
     newvar[nas] <- NA_integer_
     return(newvar)
@@ -294,14 +294,14 @@ col.w.v <- function(mat, w = NULL, bin.vars = NULL, na.rm = TRUE) {
         }
         else stop("mat must be a numeric matrix.")
     }
-    
+
     if (is_null(bin.vars)) bin.vars <- rep(FALSE, ncol(mat))
     else if (length(bin.vars) != ncol(mat) || anyNA(as.logical(bin.vars))) {
         stop("bin.vars must be a logical vector with length equal to the number of columns of mat.", call. = FALSE)
     }
     bin.var.present <- any(bin.vars)
     non.bin.vars.present <- any(!bin.vars)
-    
+
     var <- setNames(numeric(ncol(mat)), colnames(mat))
     if (is_null(w)) {
         if (non.bin.vars.present) {
@@ -411,6 +411,10 @@ mean_fast <- function(x, nas.possible = FALSE) {
     n <- length(x)
     return(s/n)
 }
+bw.nrd <- function(x) {
+    #R's bw.nrd doesn't always work, but bw.nrd0 does
+    bw.nrd0(x)*1.06/.9
+}
 
 #Formulas
 is.formula <- function(f, sides = NULL) {
@@ -428,7 +432,7 @@ subbars <- function(term) {
         term[[2]] <- subbars(term[[2]])
         return(term)
     }
-    
+
     if (is.call(term) && (term[[1]] == as.name("|") || term[[1]] == as.name("||"))) {
         term[[1]] <- as.name("+")
     }
@@ -439,7 +443,7 @@ subbars <- function(term) {
 #treat/covs
 get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep = "", ...) {
     A <- list(...)
-    
+
     #Check if data exists
     if (is_not_null(data)) {
         if (is.data.frame(data)) {
@@ -455,13 +459,13 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
         data <- environment(f)
         data.specified <- FALSE
     }
-    
+
     env <- environment(f)
-    
+
     if (!is.formula(f)) stop("f must be a formula.")
-    
+
     eval.model.matrx <- identical(f, f <- subbars(f))
-    
+
     tryCatch(tt <- terms(f, data = data),
              error = function(e) {
                  if (conditionMessage(e) == "'.' in formula and no 'data' argument") {
@@ -469,7 +473,7 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
                  }
                  else stop(conditionMessage(e), call. = FALSE)
              })
-    
+
     #Check if response exists
     if (is.formula(tt, 2)) {
         resp.vars.mentioned <- as.character(tt)[2]
@@ -482,14 +486,14 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
             else if (is_null(test)) return(TRUE)
             else return(FALSE)
         }, logical(1L))
-        
+
         if (any(resp.vars.failed)) {
             if (is_null(A[["treat"]])) stop(paste0("The given response variable, \"", as.character(tt)[2], "\", is not a variable in ", word_list(c("data", "the global environment")[c(data.specified, TRUE)], "or"), "."), call. = FALSE)
             tt <- delete.response(tt)
         }
     }
     else resp.vars.failed <- TRUE
-    
+
     if (any(!resp.vars.failed)) {
         treat.name <- resp.vars.mentioned[!resp.vars.failed][1]
         treat <- eval(parse(text=treat.name)[[1]], data, env)
@@ -498,10 +502,10 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
         treat <- A[["treat"]]
         treat.name <- NULL
     }
-    
+
     #Check if RHS variables exist
     tt.covs <- delete.response(tt)
-    
+
     rhs.vars.mentioned.lang <- attr(tt.covs, "variables")[-1]
     rhs.vars.mentioned <- vapply(rhs.vars.mentioned.lang, deparse, character(1L))
     rhs.vars.failed <- vapply(rhs.vars.mentioned, function(v) {
@@ -513,21 +517,21 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
         else if (is_null(test)) return(TRUE)
         else return(FALSE)
     }, logical(1L))
-    
+
     if (any(rhs.vars.failed)) {
         stop(paste0(c("All variables in formula must be variables in data or objects in the global environment.\nMissing variables: ",
                       paste(rhs.vars.mentioned[rhs.vars.failed], collapse=", "))), call. = FALSE)
-        
+
     }
-    
+
     rhs.term.labels <- attr(tt.covs, "term.labels")
     rhs.term.orders <- attr(tt.covs, "order")
-    
+
     rhs.df <- vapply(rhs.vars.mentioned, function(v) {
         is_(try(eval(parse(text=v)[[1]], data, env), silent = TRUE),
             c("data.frame", "matrix", "rms"))
     }, logical(1L))
-    
+
     if (any(rhs.df)) {
         if (any(rhs.vars.mentioned[rhs.df] %in% unlist(lapply(rhs.term.labels[rhs.term.orders > 1], function(x) strsplit(x, ":", fixed = TRUE))))) {
             stop("Interactions with data.frames are not allowed in the input formula.", call. = FALSE)
@@ -542,18 +546,18 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
             return(as.data.frame(df))
         }),
         rhs.vars.mentioned[rhs.df])
-        
+
         for (i in rhs.term.labels[rhs.term.labels %in% rhs.vars.mentioned[rhs.df]]) {
             ind <- which(rhs.term.labels == i)
             rhs.term.labels <- append(rhs.term.labels[-ind],
                                       values = names(addl.dfs[[i]]),
                                       after = ind - 1)
         }
-        
+
         if (data.specified) data <- do.call("cbind", unname(c(addl.dfs, list(data))))
         else data <- do.call("cbind", unname(addl.dfs))
     }
-    
+
     if (is_null(rhs.term.labels)) {
         new.form <- as.formula("~ 1")
         tt.covs <- terms(new.form)
@@ -570,22 +574,22 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
             }
             else x
         } , character(1L)), collapse = " + "))
-        
+
         new.form <- as.formula(new.form.char)
         tt.covs <- terms(new.form)
         attr(tt.covs, "intercept") <- 0
-        
+
         #Get model.frame, report error
         mf.covs <- quote(stats::model.frame(tt.covs, data,
                                             drop.unused.levels = TRUE,
                                             na.action = "na.pass"))
-        
+
         tryCatch({covs <- eval(mf.covs)},
                  error = function(e) {stop(conditionMessage(e), call. = FALSE)})
-        
+
         if (is_not_null(treat.name) && treat.name %in% names(covs)) stop("The variable on the left side of the formula appears on the right side too.", call. = FALSE)
     }
-    
+
     if (eval.model.matrx) {
         if (s <- !identical(sep, "")) {
             if (!is.character(sep) || length(sep) > 1) stop("sep must be a string of length 1.", call. = FALSE)
@@ -598,12 +602,12 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
                 }
             }
         }
-        
+
         #Get full model matrix with interactions too
         covs.matrix <- model.matrix(tt.covs, data = covs,
                                     contrasts.arg = lapply(Filter(is.factor, covs),
                                                            contrasts, contrasts=FALSE))
-        
+
         if (s) {
             for (i in names(covs)) {
                 if (is.factor(covs[[i]])) {
@@ -615,9 +619,9 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
     else {
         covs.matrix <- NULL
     }
-    
+
     if (!terms) attr(covs, "terms") <- NULL
-    
+
     return(list(reported.covs = covs,
                 model.covs = covs.matrix,
                 treat = treat,
@@ -626,7 +630,7 @@ get.covs.and.treat.from.formula <- function(f, data = NULL, terms = FALSE, sep =
 assign.treat.type <- function(treat) {
     #Returns treat with treat.type attribute
     nunique.treat <- nunique(treat)
-    
+
     if (nunique.treat < 2) {
         stop("The treatment must have at least two unique values.", call. = FALSE)
     }
@@ -677,7 +681,7 @@ nunique <- function(x, nmax = NA, na.rm = TRUE) {
         if (is.factor(x)) return(nlevels(x))
         else return(length(unique(x, nmax = nmax)))
     }
-    
+
 }
 nunique.gt <- function(x, n, na.rm = TRUE) {
     if (missing(n)) stop("n must be supplied.")
@@ -699,13 +703,51 @@ is_binary <- function(x, na.rm = TRUE) {
     if (na.rm && anyNA(x)) x <- x[!is.na(x)]
     !all_the_same(x) && all_the_same(x[x != x[1]])
 }
+is_binary_col <- function(dat, na.rm = TRUE) {
+    if (length(dim(dat)) != 2) stop("is_binary_col cannot be used with objects that don't have 2 dimensions.")
+    apply(dat, 2, is_binary)
+}
 
 #R Processing
+make_list <- function(n) {
+    if (length(n) == 1L && is.numeric(n)) {
+        vector("list", as.integer(n))
+    }
+    else if (is_(n, c("atomic", "factor"))) {
+        setNames(vector("list", length(n)), as.character(n))
+    }
+}
+ifelse_ <- function(...) {
+    if (...length() %% 2 == 0) stop("ifelse_ must have an odd number of arguments: pairs of test/yes, and one no.")
+    out <- ...elt(...length())
+    if (...length() > 1) {
+        if (!is_(out, c("atomic", "factor"))) stop("The last entry to ifelse_ must be atomic or factor.")
+        if (length(out) == 1) out <- rep(out, length(..1))
+        n <- length(out)
+        for (i in seq_len((...length() - 1)/2)) {
+            test <- ...elt(2*i - 1)
+            yes <- ...elt(2*i)
+            if (length(yes) == 1) yes <- rep(yes, n)
+            if (length(yes) != n || length(test) != n) stop("All entries must have the same length.")
+            if (!is.logical(test)) stop(paste("The", ordinal(2*i - 1), "entry to ifelse_ must be logical."))
+            if (!is_(yes, c("atomic", "factor"))) stop(paste("The", ordinal(2*i), "entry to ifelse_ must be atomic or factor."))
+            pos <- which(test)
+            out[pos] <- yes[pos]
+        }
+    }
+    else {
+        if (!is_(out, c("atomic", "factor"))) stop("The first entry to ifelse_ must be atomic or factor.")
+    }
+    return(out)
+}
 is_ <- function(x, types, stop = FALSE, arg.to = FALSE) {
     s1 <- deparse(substitute(x))
     if (is_not_null(x)) {
         for (i in types) {
             if (i == "list") it.is <- is.vector(clear_attr(x), "list")
+            else if (is_not_null(get0(paste0("is_", i)))) {
+                it.is <- get0(paste0("is_", i))(x)
+            }
             else if (is_not_null(get0(paste.("is", i)))) {
                 it.is <- get0(paste.("is", i))(x)
             }
@@ -714,7 +756,7 @@ is_ <- function(x, types, stop = FALSE, arg.to = FALSE) {
         }
     }
     else it.is <- FALSE
-    
+
     if (stop) {
         if (!it.is) {
             s0 <- ifelse(arg.to, "The argument to ", "")
@@ -723,22 +765,20 @@ is_ <- function(x, types, stop = FALSE, arg.to = FALSE) {
             stop(paste0(s0, s1, " must be a ", word_list(types, and.or = "or"), " ", s2, "."), call. = FALSE)
         }
     }
-    else {
-        return(it.is)
-    }
+    return(it.is)
 }
 is_null <- function(x) length(x) == 0L
 is_not_null <- function(x) !is_null(x)
 if_null_then <- function(x1 = NULL, x2 = NULL, ...) {
     if (is_not_null(x1)) x1
     else if (is_not_null(x2)) x2
-    else {
-        for (k in ...length()) {
+    else if (...length() > 0) {
+        for (k in seq_len(...length())) {
             if (is_not_null(...elt(k))) return(...elt(k))
         }
         return(..1)
     }
-    
+    else return(x1)
 }
 clear_null <- function(x) {
     x[vapply(x, is_null, logical(1L))] <- NULL
@@ -775,38 +815,42 @@ match_arg <- function(arg, choices, several.ok = FALSE) {
     if (missing(arg))
         stop("No argument was supplied to match_arg.", call. = FALSE)
     arg.name <- deparse(substitute(arg))
-    
+
     if (missing(choices)) {
         formal.args <- formals(sys.function(sysP <- sys.parent()))
         choices <- eval(formal.args[[as.character(substitute(arg))]],
                         envir = sys.frame(sysP))
     }
-    
+
     if (is.null(arg))
         return(choices[1L])
     else if (!is.character(arg))
-        stop(paste0("'", arg.name, "' must be NULL or a character vector"), call. = FALSE)
+        stop(paste0("The argument to '", arg.name, "' must be NULL or a character vector"), call. = FALSE)
     if (!several.ok) {
         if (identical(arg, choices))
             return(arg[1L])
         if (length(arg) > 1L)
-            stop(paste0("'", arg.name, "' must be of length 1"), call. = FALSE)
+            stop(paste0("The argument to '", arg.name, "' must be of length 1"), call. = FALSE)
     }
     else if (is_null(arg))
-        stop(paste0("'", arg.name, "' must be of length >= 1"), call. = FALSE)
-    
+        stop(paste0("The argument to '", arg.name, "' must be of length >= 1"), call. = FALSE)
+
     i <- pmatch(arg, choices, nomatch = 0L, duplicates.ok = TRUE)
     if (all(i == 0L))
-        stop(paste0("'", arg.name, "' should be one of ", word_list(choices, and.or = "or", quotes = TRUE), "."),
+        stop(paste0("The argument to '", arg.name, "' should be ", if (length(choices) > 1) {if (several.ok) "at least one of " else "one of "} else "", 
+                    word_list(choices, and.or = "or", quotes = TRUE), "."),
              call. = FALSE)
     i <- i[i > 0L]
     if (!several.ok && length(i) > 1)
-        stop("there is more than one match in 'match_arg'")
+        stop("There is more than one match in 'match_arg'")
     choices[i]
 }
 last <- function(x) {
     x[[length(x)]]
 }
+`last<-` <- function(x, value) {
+    `[[<-`(x, length(x), value)
+}
 len <- function(x, recursive = TRUE) {
     if (is.vector(x, "list") && recursive) sapply(x, len)
     else if (length(dim(x)) > 1) NROW(x)
@@ -816,3 +860,19 @@ na.rem <- function(x) {
     #A faster na.omit for vectors
     x[!is.na(x)]
 }
+check.package <- function(package.name, alternative = FALSE) {
+    packages.not.installed <- package.name[!vapply(package.name, requireNamespace, logical(1L),
+                                                  quietly = TRUE)]
+    if (is_not_null(packages.not.installed)) {
+        if (alternative) return(FALSE)
+        else {
+            plural <- length(packages.not.installed) > 1
+            stop(paste0("Package", if (plural) "s " else " ",
+                        word_list(packages.not.installed, quotes = TRUE, is.are = TRUE),
+                        " needed for this function to work. Please install ",
+                        if (plural) "them" else "it","."),
+                 call. = FALSE)
+        }
+    }
+    else return(invisible(TRUE))
+}
