@@ -273,7 +273,7 @@ get.w.npCBPS <- function(x, ...) {
     return(x$weights)
 }
 get.w.CBMSM <- function(x, ...) {
-    return(x$weights)
+    return(x$weights[sort(unique(x$id))])
 }
 get.w.ebalance <- function(x, treat, ...) {
     if (missing(treat)) stop("treat must be specified.", call. = FALSE)
@@ -293,6 +293,21 @@ get.w.optmatch <- function(x, ...) {
     treat <- as.numeric(attr(x, "contrast.group"))
     return(strata2weights(x, treat = treat))
 }
+get.w.cem.match <- function(x, ...) {
+    A <- list(...)
+    if (isTRUE(A[["use.match.strata"]])) {
+        if (is_(x, "cem.match.list")) {
+            return(unlist(lapply(x[vapply(x, is_, logical(1L), "cem.match")], function(cm) strata2weights(cm[["mstrata"]], treat = cm[["groups"]], focal = cm[["baseline.group"]])), use.names = FALSE))
+        }
+        else return(strata2weights(x[["mstrata"]], treat = x[["groups"]], focal = x[["baseline.group"]]))
+    }
+    else {
+        if (is_(x, "cem.match.list")) {
+            return(unlist(lapply(x[vapply(x, is_, logical(1L), "cem.match")], `[[`, "w"), use.names = FALSE))
+        }
+        else return(x[["w"]])
+    }
+}
 get.w.weightit <- function(x, s.weights = FALSE, ...) {
     if (s.weights) return(x$weights * x$s.weights)
     else return(x$weights)
@@ -320,4 +335,7 @@ get.w.wimids <- function(x, ...) {
     weights <- unlist(lapply(x[["models"]][-1], function(m) get.w.weightit(m)))
     weights[is.na(weights)] <- 0
     return(weights)
+}
+get.w.sbwcau <- function(x, ...) {
+    return(x[["dat_weights"]][["weights"]])
 }
\ No newline at end of file
